@@ -199,7 +199,7 @@ time_stats_col_name_text = ["Date", "Count", "Runtime (sec)"]
 time_stats_col_size = [30, 20, 20]
 time_host_stats_col_name_csv = ["stat_type", "date", "host", "count", "runtime (sec)"]
 time_host_stats_col_name_text = ["Date", "Host", "Count", "Runtime (sec)"]
-time_host_stats_col_size = [23, 25, 10, 20]
+time_host_stats_col_size = [23, 30, 10, 20]
 
 
 class JobStatistics:
@@ -2002,6 +2002,9 @@ def main():
     if "tf_stats" in sl:
         calc_tf_stats = True
 
+    if "ti_stats" in sl:
+        calc_ti_stats = True
+
     if "int_stats" in sl:
         calc_int_stats = True
 
@@ -2060,7 +2063,7 @@ def main():
                     )
                 else:
                     # int(True) -> 1
-                    tmp = sum([int(use_pmc(dir)) for dir in submit_dir])
+                    tmp = sum(int(use_pmc(dir)) for dir in submit_dir)
 
                     # All workflow are either PMC or non PMC workflows?
                     if tmp == len(submit_dir) or tmp == 0:
