@@ -66,13 +66,14 @@ def run(host="localhost", port=5000, debug=True, verbose=logging.INFO, **kwargs)
         app.config.update(DEBUG=True)
         logging.getLogger().setLevel(logging.DEBUG)
 
-    if "PEGASUS_SERVICE_URL_PREFIX" in os.environ:
+    pegasus_service_url_prefix = os.environ.get(
+        "PEGASUS_SERVICE_URL_PREFIX", app.config.get("PEGASUS_SERVICE_URL_PREFIX", None)
+    )
+    if pegasus_service_url_prefix:
         logging.info(
-            "Using non-standard URL prefix: {}".format(
-                os.environ["PEGASUS_SERVICE_URL_PREFIX"]
-            )
+            "Using non-standard URL prefix: {}".format(pegasus_service_url_prefix)
         )
-        app.config["APPLICATION_ROOT"] = os.environ["PEGASUS_SERVICE_URL_PREFIX"]
+        app.config["APPLICATION_ROOT"] = pegasus_service_url_prefix
 
     pegasusdir = os.path.expanduser("~/.pegasus")
     if not os.path.isdir(pegasusdir):
@@ -90,21 +91,29 @@ def run(host="localhost", port=5000, debug=True, verbose=logging.INFO, **kwargs)
     if os.getuid() != 0:
         log.warning("Service not running as root: Will not be able to switch users")
 
-    if "PEGASUS_SERVICE_URL_PREFIX" in os.environ:
+    options = {}
+    if app.config.get("MAX_PROCESSES", None):
+        options = {
+            "threaded": False,
+            "processes": app.config["MAX_PROCESSES"],
+            "use_reloader": False,
+        }
+    else:
+        options = {
+            "threaded": True,
+            "use_reloader": True,
+        }
+
+    if pegasus_service_url_prefix:
         application = DispatcherMiddleware(
             flask.Flask("dummy_app"), {app.config["APPLICATION_ROOT"]: app}
         )
         run_simple(
-            host,
-            port,
-            application,
-            threaded=True,
-            ssl_context=ssl_context,
-            use_reloader=True,
+            host, port, application, ssl_context=ssl_context, **options,
         )
     else:
         app.run(
-            host=host, port=port, threaded=True, ssl_context=ssl_context,
+            host=host, port=port, ssl_context=ssl_context, **options,
         )
 
     log.info("Exiting")
